# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import shutil
import tempfile

import autopilot.logging
import fixtures


logger = logging.getLogger(__name__)


class TemporaryFileInDirectory(fixtures.Fixture):
    """Create a temporafy file in a specified directory."""

    def __init__(self, parent_directory):
        super(TemporaryFileInDirectory, self).__init__()
        self.parent_directory = parent_directory

    def setUp(self):
        super(TemporaryFileInDirectory, self).setUp()
        _, self.path = tempfile.mkstemp(
            prefix='tmpfm', dir=self.parent_directory)
        logger.debug(
            'Created temporary file {} in {}.'.format(
                self.path, self.parent_directory))
        self.addCleanup(self.delete_file, self.path)

    @autopilot.logging.log_action(logger.info)
    def delete_file(self, path):
        """Delete a file, if it exists."""
        if os.path.exists(path):
            logger.debug('Deleting file.')
            os.remove(path)
        else:
            logger.debug('File does not exist.')


class TemporaryDirectoryInDirectory(fixtures.Fixture):
    """Create a temporary directory in a specified directory."""

    def __init__(self, parent_directory):
        super(TemporaryDirectoryInDirectory, self).__init__()
        self.parent_directory = parent_directory

    def setUp(self):
        super(TemporaryDirectoryInDirectory, self).setUp()
        self.path = tempfile.mkdtemp(
            prefix='tmpfm', dir=self.parent_directory)
        logger.debug(
            'Created temporary directory {} in parent directory {}'.format(
                self.path, self.parent_directory))
        self.addCleanup(self.delete_directory, self.path)

    @autopilot.logging.log_action(logger.info)
    def delete_directory(self, path):
        """Delete a directory, if it exists."""
        if os.path.exists(path):
            logger.debug('Deleting directory.')
            shutil.rmtree(path)
        else:
            logger.debug('Directory does not exist.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""File Manager app autopilot tests."""

import unittest
import logging
import mimetypes

import os

from autopilot import process
from autopilot.platform import model
from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals

from filemanager.tests import FileManagerTestCase

logger = logging.getLogger(__name__)


class TestFolderListPage(FileManagerTestCase):
    """Tests the Folder List  page features"""

    scenarios = [
        ('zip',
            {'file_to_extract': 'Test.zip',
             'extracted_dir_name': 'Test',
             'extracted_text_file_name': 'CodeOfConduct.txt',
             'extracted_image_dir_name': 'images',
             'extracted_image_name': 'ubuntu.jpg'
             }),

        ('tar',
            {'file_to_extract': 'Test.tar',
             'extracted_dir_name': 'Test',
             'extracted_text_file_name': 'CodeOfConduct.txt',
             'extracted_image_dir_name': 'images',
             'extracted_image_name': 'ubuntu.jpg'
             }),

        ('tar_gz',
            {'file_to_extract': 'Test.tar.gz',
             'extracted_dir_name': 'Test',
             'extracted_text_file_name': 'CodeOfConduct.txt',
             'extracted_image_dir_name': 'images',
             'extracted_image_name': 'ubuntu.jpg'
             }),

        ('tar_bz2',
            {'file_to_extract': 'Test.tar.bz2',
             'extracted_dir_name': 'Test',
             'extracted_text_file_name': 'CodeOfConduct.txt',
             'extracted_image_dir_name': 'images',
             'extracted_image_name': 'ubuntu.jpg'
             })
    ]

    def _get_file_by_name(self, name):
        folder_list_page = self.app.main_view.get_folder_list_page()
        fileDelegate = lambda: folder_list_page.get_file_by_name(name)
        self.assertThat(
            fileDelegate,
            Eventually(NotEquals(None)))
        return folder_list_page.get_file_by_name(name)

    def _get_file_by_index(self, index):
        folder_list_page = self.app.main_view.get_folder_list_page()
        fileDelegate = lambda: folder_list_page.get_file_by_index(index)
        self.assertThat(
            fileDelegate,
            Eventually(NotEquals(None)))
        return folder_list_page.get_file_by_index(index)

    def _go_to_location(self, location):
        # go to specified location
        # on wide UI display, we get the location dialog
        # on phone UI display, we get places popover
        device = model()
        if self.app.main_view.showSidebar:
            logger.debug("Using goto to goto %s on %s" % (location, device))
            (self.app.main_view.get_folder_list_page().get_pathbar()
                .go_to_location())
            goto_location = self.app.main_view.get_dialog()
        else:
            logger.debug("Using places to goto %s on %s" % (location, device))
            self._safe_open_popover(self.app.main_view.open_places)
            goto_location = self.app.main_view.get_popover()
        goto_location.enter_text(location)
        goto_location.ok()

    def _open_directory(self, item):
        expected_path = item.filePath
        list_view = item.list_view

        # item.open_directory()
        self.app.pointing_device.click_object(item)
        self.assertThat(
            list_view.get_current_path, Eventually(Equals(expected_path)))

    def _check_popover_exists(self):
        """Boolean, checks if popover exists."""
        try:
            popover = \
                self.app.main_view.wait_select_single('ActionSelectionPopover')
            if popover:
                return True
        except:
                try:
                    popover = self.app.main_view.wait_select_single('Popover')
                    if popover:
                        return True
                except:
                    return False

    def _safe_open_popover(self, popover_open_function):
        timeout = 0
        popover_exists = False
        while timeout < 3 and not popover_exists:
            popover_open_function()
            popover_exists = self._check_popover_exists()
            timeout += 1

    def _do_action_on_file(self, file_, action):
        logger.debug("Performing %s on file %s" % (action, file_))
        self._safe_open_popover(file_.open_actions_popover)
        file_actions_popover = self.app.main_view.get_file_actions_popover()
        file_actions_popover.click_button_by_text(action)

    def _cancel_confirm_dialog(self):
        confirm_dialog = self.app.main_view.get_confirm_dialog()
        confirm_dialog.cancel()

    def _confirm_dialog(self, text=None):
        confirm_dialog = self.app.main_view.get_confirm_dialog()
        if text:
            confirm_dialog.enter_text(text)
        confirm_dialog.ok()

    def _create_file(self):
        dir_path = os.getenv('HOME')
        self.app.main_view.copy_file_from_source_dir(
            dir_path, self.file_to_extract)

    # We can't do this testcase on phablet devices because of a lack of
    # Mir backend in autopilot
    # see https://bugs.launchpad.net/autopilot/+bug/1209004
    @unittest.skip("Can't do this properly on desktop or phablet")
    def test_open_file(self):
        self.make_file_in_home()

        first_file = self._get_file_by_index(0)
        self.app.pointing_device.click_object(first_file)

        dialog = self.app.main_view.get_file_action_dialog()
        dialog.visible.wait_for(True)

        process_manager = process.ProcessManager.create()
        original_apps = process_manager.get_running_applications()

        dialog.open()
        # make sure the dialog is open
        self.app.main_view.get_file_action_dialog()
        # Filtering copied from
        # AutopilotTestCase._compare_system_with_app_snapshot.
        current_apps = self.process_manager.get_running_applications()
        new_apps = filter(
            lambda i: i not in original_apps, current_apps)
        # Assert that only one window was opened.
        self.assertEqual(len(new_apps), 1)
        new_app = new_apps[0]
        self.assertEqual(len(new_app.get_windows()), 1)

        # TODO assert that the file was opened on the right
        # application. This depends on what's the default application
        # to open a text file. Maybe we can get this information
        # with XDG. --elopio - 2013-07-25

        # Close the opened window.
        window = new_app.get_windows()[0]
        window.close()

    def test_open_directory(self):
        dir_path = self.make_directory_in_home()
        first_dir = self._get_file_by_name(os.path.basename(dir_path))

        self._open_directory(first_dir)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self.assertThat(
            folder_list_page.get_current_path, Eventually(Equals(dir_path)))
        self._assert_number_of_files(0, home=False)
        # TODO check the label that says the directory is empty.
        # --elopio - 2013-07-25

    def test_list_folder_contents(self):
        dir_path = self.make_directory_in_home()
        dir_name = os.path.basename(dir_path)
        file_path = self.make_file_in_home()
        file_name = os.path.basename(file_path)

        self._assert_number_of_files(2)

        dir_ = self._get_file_by_name(dir_name)
        self.assertThat(dir_.fileName, Eventually(Equals(dir_name)))

        file_ = self._get_file_by_name(file_name)
        self.assertThat(file_.fileName, Eventually(Equals(file_name)))

    def test_cancel_rename_directory(self):
        dir_path = self.make_directory_in_home()
        dir_name = os.path.basename(dir_path)

        first_dir = self._get_file_by_name(dir_name)
        self._do_action_on_file(first_dir, action='Rename')
        self._cancel_confirm_dialog()

        self.assertThat(
            self.app.main_view.confirm_dialog_exists,
            Eventually(Equals(False)))
        self.assertThat(
            lambda: first_dir.fileName, Eventually(Equals(dir_name)))

    def test_cancel_rename_file(self):
        file_path = self.make_file_in_home()
        file_name = os.path.basename(file_path)

        first_file = self._get_file_by_name(file_name)
        self._do_action_on_file(first_file, action='Rename')
        self._cancel_confirm_dialog()

        self.assertThat(
            self.app.main_view.confirm_dialog_exists,
            Eventually(Equals(False)))
        self.assertThat(
            lambda: first_file.fileName,
            Eventually(Equals(file_name)))

    def test_cancel_delete_directory(self):
        dir_name = os.path.basename(self.make_directory_in_home())
        first_dir = self._get_file_by_name(dir_name)

        self._do_action_on_file(first_dir, 'Delete')
        self._cancel_confirm_dialog()

        self._assert_number_of_files(1)

    def test_cancel_delete_file(self):
        file_name = os.path.basename(self.make_file_in_home())
        first_file = self._get_file_by_name(file_name)

        self._do_action_on_file(first_file, 'Delete')
        self._cancel_confirm_dialog()

        self._assert_number_of_files(1)

    def test_create_directory(self):
        dir_name = 'Test Directory'

        self.app.main_view.click_header_action('createFolder')
        self._confirm_dialog(dir_name)

        self._assert_number_of_files(1)

        dir_ = self._get_file_by_name(dir_name)
        self.assertThat(dir_.fileName, Eventually(Equals(dir_name)))

    def test_cancel_create_directory(self):
        self.app.main_view.click_header_action('createFolder')
        self._cancel_confirm_dialog()

        self._assert_number_of_files(0)

    def test_show_directory_properties_from_list(self):
        dir_path = self.make_directory_in_home()
        dir_name = os.path.basename(dir_path)
        first_dir = self._get_file_by_name(dir_name)

        self._do_action_on_file(first_dir, 'Properties')
        file_details_popover = self.app.main_view.get_file_details_popover()
        self.assertThat(file_details_popover.visible, Eventually(Equals(True)))
        self.assertThat(
            file_details_popover.get_path, Eventually(Equals(dir_path)))
        # TODO check the rest of the fields. --elopio - 2013-07-25
        # TODO missing test, show directory properties from toolbar.
        # --elopio - 2013-07-25

    def test_show_file_properties(self):
        file_path = self.make_file_in_home()
        file_name = os.path.basename(file_path)
        first_file = self._get_file_by_name(file_name)

        self._do_action_on_file(first_file, 'Properties')
        file_details_popover = self.app.main_view.get_file_details_popover()
        self.assertThat(file_details_popover.visible, Eventually(Equals(True)))
        self.assertThat(
            file_details_popover.get_path, Eventually(Equals(file_path)))
        # TODO check the rest of the fields. --elopio - 2013-07-25

    def test_copy_directory(self):
        # Set up a directory to copy and a directory to copy it into.
        destination_dir_path = self.make_directory_in_home()
        destination_dir_name = os.path.basename(destination_dir_path)
        dir_to_copy_path = self.make_directory_in_home()
        dir_to_copy_name = os.path.basename(dir_to_copy_path)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self._assert_number_of_files(2)

        # Copy the directory.
        dir_to_copy = folder_list_page.get_file_by_name(dir_to_copy_name)
        self._do_action_on_file(dir_to_copy, 'Copy')

        # Go to the destination directory.
        destination_dir = folder_list_page.get_file_by_name(
            destination_dir_name)
        self._open_directory(destination_dir)

        # Paste the directory.
        self.app.main_view.click_header_action('paste')

        # Check that the directory is there.
        self._assert_number_of_files(1, home=False)
        first_dir = self._get_file_by_name(dir_to_copy_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(dir_to_copy_name)))

        # Go back.
        self.app.main_view.click_back()

        # Check that the directory is still there.
        self._assert_number_of_files(2)

    def test_cut_directory(self):
        # Set up a directory to cut and a directory to move it into.
        destination_dir_path = self.make_directory_in_home()
        destination_dir_name = os.path.basename(destination_dir_path)
        dir_to_cut_path = self.make_directory_in_home()
        dir_to_cut_name = os.path.basename(dir_to_cut_path)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self._assert_number_of_files(2)

        # Cut the directory.
        dir_to_cut = folder_list_page.get_file_by_name(dir_to_cut_name)
        self._do_action_on_file(dir_to_cut, 'Cut')

        # Go to the destination directory.
        destination_dir = folder_list_page.get_file_by_name(
            destination_dir_name)
        self._open_directory(destination_dir)

        # Paste the directory.
        self.app.main_view.click_header_action('paste')

        # Check that the directory is there.
        self._assert_number_of_files(1, home=False)
        first_dir = self._get_file_by_name(dir_to_cut_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(dir_to_cut_name)))

        # Go back.
        self.app.main_view.click_back()

        # Check that the directory is not there.
        self._assert_number_of_files(1)
        first_dir = self._get_file_by_name(destination_dir_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(destination_dir_name)))

    def test_copy_file(self):
        # Set up a file to copy and a directory to copy it into.
        destination_dir_path = self.make_directory_in_home()
        destination_dir_name = os.path.basename(destination_dir_path)
        file_to_copy_path = self.make_file_in_home()
        file_to_copy_name = os.path.basename(file_to_copy_path)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self._assert_number_of_files(2)

        # Copy the file.
        file_to_copy = folder_list_page.get_file_by_name(file_to_copy_name)
        self._do_action_on_file(file_to_copy, 'Copy')

        # Go to the destination directory.
        destination_dir = folder_list_page.get_file_by_name(
            destination_dir_name)
        self._open_directory(destination_dir)

        # Paste the file.
        self.app.main_view.click_header_action('paste')

        # Check that the file is there.
        self._assert_number_of_files(1)
        first_dir = self._get_file_by_name(file_to_copy_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(file_to_copy_name)))

        # Go back.
        self.app.main_view.click_back()

        # Check that the file is still there.
        self._assert_number_of_files(2)
        first_dir = self._get_file_by_name(destination_dir_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(destination_dir_name)))

    def test_cut_file(self):
        # Set up a file to cut and a directory to move it into.
        destination_dir_path = self.make_directory_in_home()
        destination_dir_name = os.path.basename(destination_dir_path)
        file_to_cut_path = self.make_file_in_home()
        file_to_cut_name = os.path.basename(file_to_cut_path)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self._assert_number_of_files(2)

        # Cut the file.
        file_to_cut = folder_list_page.get_file_by_name(file_to_cut_name)
        self._do_action_on_file(file_to_cut, 'Cut')

        # Go to the destination directory.
        destination_dir = folder_list_page.get_file_by_name(
            destination_dir_name)
        self._open_directory(destination_dir)

        # Paste the file.
        self.app.main_view.click_header_action('paste')

        # Check that the file is there.
        self._assert_number_of_files(1, home=False)
        first_dir = self._get_file_by_name(file_to_cut_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(file_to_cut_name)))

        # Go back.
        self.app.main_view.click_back()

        # Check that the file is not there.
        self._assert_number_of_files(1)
        first_dir = self._get_file_by_name(destination_dir_name)
        self.assertThat(
            first_dir.fileName, Eventually(Equals(destination_dir_name)))

    def test_go_back(self):
        dir_name = os.path.basename(self.make_directory_in_home())
        first_dir = self._get_file_by_name(dir_name)
        self._open_directory(first_dir)

        self.app.main_view.click_back()

        folder_list_page = self.app.main_view.get_folder_list_page()
        self.assertThat(
            folder_list_page.get_current_path,
            Eventually(Equals(self.fakehome)))

    def test_extract_archive_file(self):
        """Test that extracts an archive file from content directory."""
        self._create_file()
        self.app.main_view.go_to_place('placePath', self.file_to_extract)

        self._do_action_on_file(
            self._get_file_by_name(self.file_to_extract), 'Extract archive')
        self._confirm_dialog()
        self._assert_number_of_files(2)

        # Verify that the extraced directory name is correct
        self.assertThat(
            self._get_file_by_name(self.extracted_dir_name).text,
            Eventually(Equals(self.extracted_dir_name)))

        # Verify that the extraced text file name is a text file
        self._open_directory(self._get_file_by_name(
            self.extracted_dir_name))
        self.assertThat(
            self._get_file_by_name(self.extracted_text_file_name).text,
            Eventually(Equals(self.extracted_text_file_name)))

        self.assertThat(
            mimetypes.guess_type(self.extracted_text_file_name)[0],
            Equals('text/plain'))

        # Verify that the extraced image directory name is correct
        self.assertThat(
            self._get_file_by_name(self.extracted_image_dir_name).text,
            Eventually(Equals(self.extracted_image_dir_name)))

        # Verify that the extraced image file name is an image
        self._open_directory(self._get_file_by_name(
            self.extracted_image_dir_name))
        self.assertThat(
            self._get_file_by_name(self.extracted_image_name).text,
            Eventually(Equals(self.extracted_image_name)))

        self.assertThat(
            mimetypes.guess_type(self.extracted_image_name)[0],
            Equals('image/jpeg'))

    def test_cancel_extract_archive_file(self):
        """Test that cancels opening an archive file from content directory."""
        self._create_file()

        self.app.main_view.go_to_place('placePath', self.file_to_extract)
        self._do_action_on_file(
            self._get_file_by_name(self.file_to_extract), 'Extract archive')
        self._cancel_confirm_dialog()

        self._assert_number_of_files(1)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Filemanager app autopilot tests."""

import logging
import os
import shutil
import tempfile

import fixtures
from autopilot import logging as autopilot_logging
from filemanager import CMakePluginParser

from autopilot.matchers import Eventually
from autopilot.testcase import AutopilotTestCase
from testtools.matchers import Equals
import ubuntuuitoolkit

import filemanager
from filemanager import fixture_setup as fm_fixtures
import gi
gi.require_version('Click', '0.4')
from gi.repository import Click

logger = logging.getLogger(__name__)


class BaseTestCaseWithPatchedHome(AutopilotTestCase):

    """A common test case class that provides several useful methods for
    filemanager-app tests.

    """

    def get_launcher_and_type(self):
        if os.path.exists(self.local_location_binary):
            launcher = self.launch_test_local
            test_type = 'local'
        elif os.path.exists(self.installed_location_binary):
            launcher = self.launch_test_installed
            test_type = 'deb'
        else:
            launcher = self.launch_test_click
            test_type = 'click'
        return launcher, test_type

    def setUp(self):
        self.binary = 'filemanager'
        self.source_dir = os.path.dirname(
            os.path.dirname(os.path.abspath('.')))
        self.build_dir = self._get_build_dir()

        self.local_location = self.build_dir
        self.local_location_qml = os.path.join(self.build_dir,
                                               'src', 'app',
                                               'qml', self.binary + '.qml')
        self.local_location_binary = os.path.join(self.build_dir,
                                                  'src', 'app', self.binary)
        self.installed_location_binary = os.path.join('/usr/bin/', self.binary)
        self.installed_location_qml = \
            '/usr/share/filemanager/qml/filemanager.qml'
        super(BaseTestCaseWithPatchedHome, self).setUp()
        self.launcher, self.test_type = self.get_launcher_and_type()
        self.real_home_dir = os.getenv('HOME')
        self.patch_home()

    @autopilot_logging.log_action(logger.info)
    def launch_test_local(self):
        self.useFixture(fixtures.EnvironmentVariable(
            'QML2_IMPORT_PATH', newvalue=os.path.join(self.local_location,
                                                      'src', 'plugin')))
        return self.launch_test_application(
            self.local_location_binary,
            '-p',
            '-q', self.local_location_qml,
            app_type='qt',
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase)

    @autopilot_logging.log_action(logger.info)
    def launch_test_installed(self):
        return self.launch_test_application(
            self.installed_location_binary,
            '-q', self.installed_location_qml,
            app_type='qt',
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase)

    @autopilot_logging.log_action(logger.info)
    def launch_test_click(self):
        # We need to pass the "--forceAuth false" argument to the filemanager
        # binary, but ubuntu-app-launch doesn't pass arguments to the exec line
        # on the desktop file. So we make a test desktop file that has the
        # "--forceAuth false"  on the exec line.
        desktop_file_path = self.write_sandbox_desktop_file()
        desktop_file_name = os.path.basename(desktop_file_path)
        application_name, _ = os.path.splitext(desktop_file_name)
        return self.launch_upstart_application(
            application_name,
            emulator_base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase)

    def write_sandbox_desktop_file(self):
        desktop_file_dir = self.get_local_desktop_file_directory()
        desktop_file = self.get_named_temporary_file(
            suffix='.desktop', dir=desktop_file_dir)
        desktop_file.write('[Desktop Entry]\n')
        version, installed_path = self.get_installed_version_and_directory()
        filemanager_sandbox_exec = (
            'aa-exec-click -p com.ubuntu.filemanager_filemanager_{}'
            ' -- filemanager --forceAuth false'.format(version))
        desktop_file_dict = {
            'Type': 'Application',
            'Name': 'filemanager',
            'Exec': filemanager_sandbox_exec,
            'Icon': 'Not important',
            'Path': installed_path
        }
        for key, value in desktop_file_dict.items():
            desktop_file.write('{key}={value}\n'.format(key=key, value=value))
        desktop_file.close()
        logger.debug(filemanager_sandbox_exec)
        for key, value in desktop_file_dict.items():
            logger.debug("%s: %s" % (key, value))
        return desktop_file.name

    def get_local_desktop_file_directory(self):
        return os.path.join(
            self.real_home_dir, '.local', 'share', 'applications')

    def get_named_temporary_file(
            self, dir=None, mode='w+t', delete=False, suffix=''):
        # Discard files with underscores which look like an APP_ID to Unity
        # See https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1329141
        chars = tempfile._RandomNameSequence.characters.strip("_")
        tempfile._RandomNameSequence.characters = chars
        return tempfile.NamedTemporaryFile(
            dir=dir, mode=mode, delete=delete, suffix=suffix)

    def get_installed_version_and_directory(self):
        db = Click.DB()
        db.read()
        package_name = 'com.ubuntu.filemanager'
        registry = Click.User.for_user(db, name=os.environ.get('USER'))
        version = registry.get_version(package_name)
        directory = registry.get_path(package_name)
        return version, directory

    def patch_home(self):
        """ mock /home for testing purposes to preserve user data
        """

        # if running on non-phablet device,
        # run in temp folder to avoid mucking up home
        # bug 1316746
        # bug 1376423
        if self.test_type is not 'click':
            temp_dir_fixture = fixtures.TempDir()
            self.useFixture(temp_dir_fixture)
            temp_dir = temp_dir_fixture.path

            # before we set fixture, copy xauthority if needed
            self.useFixture(fixtures.EnvironmentVariable('HOME',
                                                         newvalue=temp_dir))

            logger.debug("Patched home to fake home directory %s" % temp_dir)

    def patch_environment(self, key, value):
        def patch_var(key, value):
            logging.info(
                "Setting environment variable '%s' to '%s'",
                key,
                value
            )

            os.environ[key] = value

        original = os.getenv(key)
        if original is None:
            original = ''
        patch_var(key, value)
        # on click, we should cleanup
        if self.test_type is 'click':
            self.addCleanup(patch_var, key, original)

    def _get_build_dir(self):
        """
        Returns the build dir after having parsed the CMake config file
        generated by Qt Creator. If it cannot find it or it cannot be parsed,
        an in-tree build is assumed and thus returned.
        """
        try:
            cmake_config = CMakePluginParser.CMakePluginParser(os.path.join(
                self.source_dir, 'CMakeLists.txt.user'))
            build_dir = cmake_config.active_build_dir
        except:
            build_dir = self.source_dir

        return build_dir


class FileManagerTestCase(BaseTestCaseWithPatchedHome):

    """Base test case that launches the filemanager-app."""

    def setUp(self):
        super(FileManagerTestCase, self).setUp()
        self.fakehome = os.getenv('HOME')
        self.original_file_count = \
            len([i for i in os.listdir(self.fakehome)
                 if not i.startswith('.')])
        logger.debug('Directory Listing for HOME\n%s' %
                     os.listdir(self.fakehome))
        logger.debug('File count in HOME is %s' % self.original_file_count)
        self.app = filemanager.Filemanager(self.launcher(), self.test_type)

    def make_file_in_home(self):
        return self.make_content_in_home('file')

    def make_directory_in_home(self):
        return self.make_content_in_home('directory')

    def make_content_in_home(self, type_):
        if type_ != 'file' and type_ != 'directory':
            raise ValueError('Unknown content type: "{0}"', type_)
        if type_ == 'file':
            temp_file = fm_fixtures.TemporaryFileInDirectory(self.fakehome)
            self.useFixture(temp_file)
            path = temp_file.path
        else:
            temp_dir = fm_fixtures.TemporaryDirectoryInDirectory(
                self.fakehome)
            self.useFixture(temp_dir)
            path = temp_dir.path

        # putting ignore_errors=True to avoid tests failing when cutting or
        # deleting file or directory has already canceled the file or directory
        self.addCleanup(shutil.rmtree, path, ignore_errors=True)

        logger.debug('Directory Listing for HOME\n%s' %
                     os.listdir(self.fakehome))
        self._assert_number_of_files(1)
        return path

    def _assert_number_of_files(self, expected_number_of_files, home=True):
        if home:
            expected_number_of_files += self.original_file_count
        folder_list_page = self.app.main_view.get_folder_list_page()
        self.assertThat(
            folder_list_page.get_number_of_files_from_list,
            Eventually(Equals(expected_number_of_files), timeout=60))
        self.assertThat(
            folder_list_page.get_number_of_files_from_header,
            Eventually(Equals(expected_number_of_files), timeout=60))



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""File Manager app autopilot emulators."""

import logging
import re
import time
import os
import shutil

import autopilot.logging
from autopilot import input
from autopilot.introspection import dbus
import ubuntuuitoolkit

logger = logging.getLogger(__name__)


class FilemanagerException(ubuntuuitoolkit.ToolkitException):

    """Exception raised when there are problems with the Filemanager."""


class Filemanager(object):

    """Autopilot helper object for the filemanager application."""

    def __init__(self, app_proxy, test_type):
        self.app = app_proxy
        self.test_type = test_type
        self.main_view = self.app.select_single(MainView)

    @property
    def pointing_device(self):
        return self.app.pointing_device


class MainView(ubuntuuitoolkit.MainView):
    """File Manager MainView Autopilot emulator."""

    def __init__(self, *args):
        super(MainView, self).__init__(*args)

    @autopilot.logging.log_action(logger.info)
    def go_to_place(self, object_name, file_to_extract):
        """Open one of the bookmarked place folders.

        :param object_name: The objectName property of the place to open.

        """

        if self.showSidebar:
            self._go_to_place_from_side_bar(object_name)
        else:
            self._go_to_place_from_places_page(object_name, file_to_extract)

    def _go_to_place_from_side_bar(self, object_name):
        dir_path = os.getenv('HOME')
        if object_name == 'placePath':
            self.click_header_action('Find')
            go_to_dialog = self.get_go_to_dialog()
            go_to_dialog.enter_text(dir_path)
            go_to_dialog.ok()
        else:
            side_bar = self.get_folder_list_page().get_sidebar()
            side_bar.go_to_place(object_name)

    def get_folder_list_page(self):
        """Return the FolderListPage emulator of the MainView."""
        page = self.wait_select_single(FolderListPage)
        page.main_view = self
        return page

    def get_places_page(self):
        return self.wait_select_single(PlacesPage)

    def _go_to_place_from_places_page(self, object_name, file_to_extract):
        dir_path = os.getenv('HOME')
        if object_name == 'placePath':
            self.copy_file_from_source_dir(dir_path, file_to_extract)
        placespage = self.open_places()
        placespage.go_to_place(object_name, dir_path)

    @autopilot.logging.log_action(logger.info)
    def open_places(self):
        if not self.showSidebar:
            self._drag_bottomedge_to_open_places()
            return self.wait_select_single(PlacesPage)
        else:
            raise FilemanagerException(
                'The places page cannot be opened on wide mode.')

    def _drag_bottomedge_to_open_places(self):
        """Bring the places page to the screen"""
        try:
            action_item = self.wait_select_single(
                'UCUbuntuShape', objectName='bottomEdgeTip')
            action_item.visible.wait_for(True)
            action_item.isAnimating.wait_for(False)
            start_x = (action_item.globalRect.x +
                       (action_item.globalRect.width * 0.5))
            start_y = (action_item.globalRect.y +
                       (action_item.height * 0.5))
            stop_y = start_y - (self.height * 0.7)
            self.pointing_device.drag(start_x, start_y,
                                      start_x, stop_y, rate=2)

        except dbus.StateNotFoundError:
            logger.error('BottomEdge element not found.')
            raise

        flickable = self.wait_select_single(
            "QQuickFlickable", objectName="placesFlickable")
        flickable.wait_until_destroyed()

    @autopilot.logging.log_action(logger.info)
    def rename(self, original_name, new_name):
        """Rename a file or directory.

        :param original_name: The name of the file or directory to rename.
        :param new_name: The new name to set for the file or directory.

        """
        actions_popover = self.open_actions(original_name)
        actions_popover.click_button_by_text('Rename')
        confirm_dialog = self.wait_select_single(ConfirmDialogWithInput)
        confirm_dialog.enter_text(new_name)
        confirm_dialog.ok()

    @autopilot.logging.log_action(logger.info)
    def open_actions(self, name):
        """Open the list of available actions of a file or directory."""
        folder_list_page = self.get_folder_list_page()
        folder_list_page.open_file_actions(name)
        actions_popover = self.get_action_selection_popover(
            'fileActionsPopover')
        actions_popover.visible.wait_for(True)
        return actions_popover

    @autopilot.logging.log_action(logger.info)
    def delete(self, name):
        """Delete a file or directory.

        :param name: The name of the file or directory to delete.

        """
        actions_popover = self.open_actions(name)
        actions_popover.click_button_by_text('Delete')
        confirm_dialog = self.wait_select_single(ConfirmDialog)
        confirm_dialog.ok()

    def get_file_actions_popover(self):
        """Return the ActionSelectionPopover emulator of the file actions."""
        return self.get_action_selection_popover('fileActionsPopover')

    def click_header_action(self, objectName):
        header = self.get_header()
        header.click_action_button(objectName)

    def click_back(self):
        header = self.get_header()
        header.click_custom_back_button()

    # TODO: can probably be removed
    def get_folder_actions_popover(self):
        """Return the ActionSelectionPopover emulator of the folder actions."""
        return self.get_action_selection_popover('folderActionsPopover')

    # TODO: can probably be removed
    def folder_actions_popover_exists(self):
        """Boolean, checks if the folder actions popover exists."""
        try:
            popover = self.get_folder_actions_popover()
            if popover:
                return True
        except:
            return False

    # TODO: can probably be removed
    def file_actions_popover_exists(self):
        """Boolean, checks if the file actions popover exists."""
        try:
            popover = self.get_file_actions_popover()
            if popover:
                return True
        except:
            return False

    def get_file_details_popover(self):
        """Return the FileDetailsPopover emulator."""
        return self.wait_select_single(FileDetailsPopover)

    def get_file_action_dialog(self):
        """Return the FileActionDialog emulator."""
        return self.wait_select_single(FileActionDialog)

    def file_action_dialog_exists(self):
        """Boolean checks if the FileActionDialog exists."""
        dialog = self.select_many(FileActionDialog)
        if dialog == '[]':
            return True
        return False

    def get_confirm_dialog(self):
        """Return a confirm dialog emulator"""
        try:
            dialog = self.wait_select_single(ConfirmDialog)
        except dbus.StateNotFoundError:
            dialog = self.wait_select_single(ConfirmDialogWithInput)
        return dialog

    def confirm_dialog_exists(self):
        """Boolean checks if a confirm dialog exists"""
        dialog = self.select_many(ConfirmDialog)
        if dialog == '[]':
            dialog = self.select_many(ConfirmDialogWithInput)
        if dialog == '[]':
            return True
        return False

    def get_dialog(self):
        """Return a dialog emulator"""
        return self.wait_select_single(Dialog)

    def get_popover(self):
        """Return a popover emulator"""
        return self.wait_select_single(Popover)

    def get_go_to_dialog(self):
        """Return a go to dialog """
        return self.wait_select_single(objectName='goToDialog')

    def copy_file_from_source_dir(self, dir_path, file_to_extract):
        content_dir_file = os.path.join(os.path.dirname(
            __file__), 'content', file_to_extract)
        shutil.copy(content_dir_file, dir_path)


class PlacesSidebar(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """PlacesSidebar Autopilot emulator."""

    @autopilot.logging.log_action(logger.info)
    def go_to_place(self, object_name):
        """Open one of the bookmarked place folders or content folder depending
        on object_name parameter

        :param object_name: The objectName property of the place to open
                            If value is inputField then open content folder

        """
        place = self.wait_select_single('Standard', objectName=object_name)
        self.pointing_device.click_object(place)


class FolderListPage(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FolderListPage Autopilot emulator."""

    def open_file_actions(self, name):
        """Open the actions menu of a file or folder.

        :param name: The name of the file or folder.

        """
        delegate = self.get_file_by_name(name)
        delegate.open_actions_popover()

    def get_files_and_folders(self):
        """Return the list of files and folders of the opened directory.

        The list returned will contain the names of the files and folders.

        """
        if self.showingListView:
            view = self.select_single(FolderListView)
        else:
            view = self.select_single(FolderIconView)
        return view.get_files_and_folders()

    def get_number_of_files_from_list(self):
        """Return the number of files shown on the folder."""
        if self.showingListView:
            return len(self.select_many(FolderListDelegate))
        else:
            return len(self.select_many(FolderIconDelegate))

    def get_file_by_index(self, index):
        """Return the FolderListDelegate emulator of the file or folder.

        :parameter index: The index of file or folder.

        """
        if self.showingListView:
            file_ = self.select_many(FolderListDelegate)[index]
        else:
            file_ = self.select_many(FolderIconDelegate)[index]
        file_.list_view = self.select_single(FolderListView)
        return file_

    def get_file_by_name(self, name):
        """Return the FolderListDelegate emulator of the file or folder.

        :parameter name: The name of the file or folder.

        """
        files = self.select_many(FolderListDelegate)
        if not self.showingListView:
            files = self.select_many(FolderIconDelegate)
        for file_ in files:
            if file_.fileName == name:
                if self.showingListView:
                    file_.list_view = self.select_single(FolderListView)
                else:
                    file_.list_view = self.select_single(FolderIconView)
                return file_
        raise ValueError(
            'File with name "{0}" not found.'.format(name))

    def get_current_path(self):
        """Get the path of the folder currently displayed."""
        if self.showingListView:
            return self.select_single(FolderListView).get_current_path()
        else:
            return self.select_single(FolderIconView).get_current_path()

    def get_number_of_files_from_header(self):
        if self.showingListView:
            return self.select_single(FolderListView).get_number_of_files()
        else:
            return self.select_single(FolderIconView).get_number_of_files()

    def get_sidebar(self):
        if self.main_view.showSidebar:
            return self.select_single(PlacesSidebar)
        else:
            raise ValueError(
                'Places sidebar is hidden in small mode.')

    def get_pathbar(self):
        if self.main_view.showSidebar:
            return self.main_view.get_toolbar().select_single(PathBar)
        else:
            raise ValueError(
                'Path bar is hidden in small mode.')


class PlacesPage(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """ Places Page Autopilot emulator.     """

    @autopilot.logging.log_action(logger.info)
    def go_to_place(self, object_name, dir_path):
        """Open one of the bookmarked place folders or content folder depending
            on object_name value

        :param object_name: The objectName property of the place to open
                            If equals inputField then open content folder

        """

        if object_name == 'placePath':
            place = self.wait_select_single(
                'TextField', objectName=object_name)
            place.write(dir_path, clear=True)
            ok_button = self.wait_select_single(
                "Button", objectName="okButton")
            self.pointing_device.click_object(ok_button)
        else:
            place = self.wait_select_single('Standard', objectName=object_name)
            self.pointing_device.click_object(place)


class FolderListView(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FolderListView Autopilot emulator."""

    SPLIT_HEADER_REGEX = '(.+) \((\d+) \w+\)$'
    # Regular expression to split the header text. The header text has the form
    # /path/to/dir (# files). So with this expression, we can split the header
    # in two groups, (.+) will match the path and (\d+) the number of files.

    def get_current_path(self):
        path, _ = self._split_header_text()
        return path

    def _split_header_text(self):
        header_text = self.select_single(
            'ListItemHeader', objectName='listViewSmallHeader').text
        match = re.match(self.SPLIT_HEADER_REGEX, header_text)
        if match:
            path = match.group(1)
            number_of_files = match.group(2)
            return path, number_of_files

    def get_number_of_files(self):
        _, number_of_files = self._split_header_text()
        return int(number_of_files)

    def get_files_and_folders(self):
        """Return the list of files and folders of the opened directory."""
        list_delegates = self.select_many(FolderListDelegate)
        # sort by y, x
        list_delegates = sorted(
            list_delegates,
            key=lambda item: (item.globalRect.y, item.globalRect.x))

        return [item.fileName for item in list_delegates]


class FolderIconView(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FolderListView Autopilot emulator."""

    SPLIT_HEADER_REGEX = '(.+) \((\d+) \w+\)$'
    # Regular expression to split the header text. The header text has the form
    # /path/to/dir (# files). So with this expression, we can split the header
    # in two groups, (.+) will match the path and (\d+) the number of files.

    def get_current_path(self):
        path, _ = self._split_header_text()
        return path

    def _split_header_text(self):
        header_text = self.select_single(
            'ListItemHeader', objectName='iconViewHeader').text
        match = re.match(self.SPLIT_HEADER_REGEX, header_text)
        if match:
            path = match.group(1)
            number_of_files = match.group(2)
            return path, number_of_files

    def get_number_of_files(self):
        _, number_of_files = self._split_header_text()
        return int(number_of_files)

    def get_files_and_folders(self):
        """Return the list of files and folders of the opened directory."""
        icon_delegates = self.select_many(FolderIconDelegate)
        # sort by y, x
        icon_delegates = sorted(
            icon_delegates,
            key=lambda icon: (icon.globalRect.y, icon.globalRect.x))

        return [icon.fileName for icon in icon_delegates]


class FolderListDelegate(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FolderListPage Autopilot emulator.

    This is a file or folder on the FolderListPage.

    """

    def __init__(self, *args):
        super(FolderListDelegate, self).__init__(*args)
        self.pointing_device = ubuntuuitoolkit.get_pointing_device()

    def open_directory(self):
        """Open the directory."""
        # TODO Check if it is a directory. If not, raise an error.
        # This is not currently possible because Autopilot is overwriting the
        # path attribute. Reported on
        # https://bugs.launchpad.net/autopilot/+bug/1205204
        # --elopio - 2013-07-25
        self.pointing_device.click_object(self)

    def open_file(self):
        # TODO For this we would need to access the FileActionDialog that's
        # child of the MainView, but that's not currently possible with
        # autopilot. Reported on
        # bug https://bugs.launchpad.net/autopilot/+bug/1195141
        # --elopio - 2013-07-25
        raise NotImplementedError()

    def open_actions_popover(self):
        """Open the actions popover of the file or folder."""
        self.pointing_device.move_to_object(self)
        self.pointing_device.press()
        time.sleep(2)
        self.pointing_device.release()


class FolderIconDelegate(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FolderIconPage Autopilot emulator.

    This is a file or folder on the FolderListPage.

    """

    def __init__(self, *args):
        super(FolderIconDelegate, self).__init__(*args)
        self.pointing_device = ubuntuuitoolkit.get_pointing_device()

    def open_directory(self):
        """Open the directory."""
        # TODO Check if it is a directory. If not, raise an error.
        # This is not currently possible because Autopilot is overwriting the
        # path attribute. Reported on
        # https://bugs.launchpad.net/autopilot/+bug/1205204
        # --elopio - 2013-07-25
        self.pointing_device.click_object(self)

    def open_file(self):
        # TODO For this we would need to access the FileActionDialog that's
        # child of the MainView, but that's not currently possible with
        # autopilot. Reported on
        # bug https://bugs.launchpad.net/autopilot/+bug/1195141
        # --elopio - 2013-07-25
        raise NotImplementedError

    def open_actions_popover(self):
        """Open the actions popover of the file or folder."""
        self.pointing_device.move_to_object(self)
        self.pointing_device.press()
        time.sleep(2)
        self.pointing_device.release()


class FileActionDialog(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FileActionDialog Autopilot emulator."""

    def __init__(self, *args):
        super(FileActionDialog, self).__init__(*args)
        self.pointing_device = ubuntuuitoolkit.get_pointing_device()

    def open(self):
        open_button = self.select_single('Button', objectName='openButton')
        self.pointing_device.click_object(open_button)

    def cancel(self):
        cancel_button = self.select_single('Button', objectName='cancelButton')
        self.pointing_device.click_object(cancel_button)


class ConfirmDialog(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """ConfirmDialog Autopilot emulator."""

    def __init__(self, *args):
        super(ConfirmDialog, self).__init__(*args)
        self.pointing_device = ubuntuuitoolkit.get_pointing_device()

    def ok(self):
        okButton = self.select_single('Button', objectName='okButton')
        self.pointing_device.click_object(okButton)

    def cancel(self):
        cancel_button = self.select_single('Button', objectName='cancelButton')
        self.pointing_device.click_object(cancel_button)


class ConfirmDialogWithInput(ConfirmDialog):
    """ConfirmDialogWithInput Autopilot emulator."""

    def __init__(self, *args):
        super(ConfirmDialogWithInput, self).__init__(*args)
        self.keyboard = input.Keyboard.create()

    def enter_text(self, text, clear=True):
        text_field = self._select_text_field()
        text_field.write(text, clear)

    def _select_text_field(self):
        return self.select_single(
            ubuntuuitoolkit.TextField, objectName='inputField')


class Dialog(ConfirmDialogWithInput):
    """Dialog Autopilot emulator."""


class Popover(ConfirmDialogWithInput):
    """Popover Autopilot emulator, containing buttons and an inputfield"""

    def click_button(self, text):
        """Click a button on the popover.

        XXX We are receiving the text because there's no way to set the
        objectName on the action. This is reported at
        https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1205144
        --elopio - 2013-07-25

        :parameter text: The text of the button.

        """
        button = self._get_button(text)
        self.pointing_device.click_object(button)

    def _get_button(self, text):
        buttons = self.select_many('Empty')
        for button in buttons:
            if button.text == text:
                return button


class FileDetailsPopover(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """FileDetailsPopover Autopilot emulator."""

    def get_path(self):
        return self.select_single('UCLabel', objectName='pathLabel').text


class PathBar(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):

    def go_to_location(self):
        editButton = self.select_single(objectName='goToButton')
        self.pointing_device.click_object(editButton)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Author:
#   David Planella <david.planella@ubuntu.com>

"""
This module parses a configuration file from the Qt Creator's CMake plugin and
enables programmatical read-only access to several of its configuration options
"""

import sys
from lxml import etree


class CMakePluginParseError(Exception):
    """
    Custom exception for errors during the parsing of a
    CMakeLists.txt.user file
    """
    def __init__(self, message):
        Exception.__init__(self, message)


class CMakePluginParser(object):
    """
    Parses a CMake plugin's config file and provides R/O access to its
    configuration options """

    def __init__(self, cmakelists_usr_file='CMakeLists.txt.user'):
        self.usr_file = cmakelists_usr_file

        try:
            self.info = etree.parse(self.usr_file)
        except:
            sys.stderr.write("Could not open the given " +
                             "CMakeLists.txt.user file: " + self.info)
            raise

    def _get_active_build_target(self):
        """
        Return the active build target from the current project in Qt Creator
        """

        try:
            active_build_target_nr = self.info.xpath(
                "./data/variable" +
                "[text()='ProjectExplorer.Project.ActiveTarget']" +
                "/../value")[0].text
        except:
            raise CMakePluginParseError("Could not find the active build " +
                                        "target in the CMake plugin's config")

        active_build_target = "ProjectExplorer.Project.Target." + \
            active_build_target_nr

        return active_build_target

    def _get_active_build_config(self, active_build_target):
        """Return the active build config from the active build targed"""

        try:
            active_build_config_nr = self.info.xpath(
                "./data/variable[text()='{0}']".format(active_build_target) +
                "/..//value[@key="
                "'ProjectExplorer.Target.ActiveBuildConfiguration']")[0].text
        except:
            raise CMakePluginParseError("Could not find the active build " +
                                        "target's active build config " +
                                        "in the CMake plugin's config")

        active_build_config = "ProjectExplorer.Target.BuildConfiguration." + \
            active_build_config_nr

        return active_build_config

    def _get_active_build_config_path(self):
        """Return the active build config's absolute path"""

        active_build_target = self._get_active_build_target()
        active_build_config = \
            self._get_active_build_config(active_build_target)

        try:
            active_build_config_node = self.info.xpath(
                "./data/variable[text()='{0}']".format(active_build_target) +
                "/..//valuemap[@key='{0}']".format(active_build_config))[0]
        except:
            raise CMakePluginParseError("Could not find the active " +
                                        "build config's node " +
                                        "in the CMake plugin's config")

        try:
            active_build_config_path = active_build_config_node.xpath(
                "./value[@key=" +
                "'ProjectExplorer.BuildConfiguration.BuildDirectory']")[0].text
        except:
            raise CMakePluginParseError("Could not find the active build " +
                                        "directory in the CMake plugin's " +
                                        "config")

        return active_build_config_path

    @property
    def active_build_dir(self):
        """Return the active build config's directory as an absolute path"""
        return self._get_active_build_config_path()



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
from filemanager.tests import FileManagerTestCase


class ContextMenuTestCase(FileManagerTestCase):
    """Test cases for the context menu of the file manager app."""

    def get_current_directory_files_and_folders(self):
        folder_list_page = self.app.main_view.get_folder_list_page()
        return folder_list_page.get_files_and_folders()

    def test_rename_directory(self):
        """Test renaming a directory.

        The name of the directory must be updated on the list of folder
        contents.

        """
        original_directory = os.path.basename(self.make_directory_in_home())
        new_name = original_directory + 'edit'

        self.app.main_view.rename(original_directory, new_name)

        files_and_folders = self.get_current_directory_files_and_folders()
        self.assertTrue(new_name in files_and_folders)

    def test_rename_file(self):
        """Test renaming a file.

        The name of the file must be updated on the list of folder contents.

        """
        original_file = os.path.basename(self.make_file_in_home())
        new_name = original_file + 'edit'

        self.app.main_view.rename(original_file, new_name)

        files_and_folders = self.get_current_directory_files_and_folders()
        self.assertTrue(new_name in files_and_folders)

    def test_delete_directory(self):
        """Test deleting a directory.

        The directory must no longer be displayed on the list of folder
        contents.

        """
        dir_name = os.path.basename(self.make_directory_in_home())

        self.app.main_view.delete(dir_name)

        files_and_folders = self.get_current_directory_files_and_folders()
        self.assertFalse(dir_name in files_and_folders)

    def test_delete_file(self):
        """Test deleting a file.

        The file must no longer be displayed on the list of folder contents.

        """
        file_name = os.path.basename(self.make_file_in_home())

        self.app.main_view.delete(file_name)

        files_and_folders = self.get_current_directory_files_and_folders()
        self.assertFalse(file_name in files_and_folders)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from filemanager.tests import FileManagerTestCase


class PlacesTestCase(FileManagerTestCase):
    """Test cases for the places section of the file manager app."""

    def test_go_home_must_open_the_home_directory(self):
        """Test that opens the Home bookmark from the places section."""
        self.app.main_view.go_to_place('placeHome', None)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self.assertThat(
            folder_list_page.get_current_path,
            Eventually(Equals(self.fakehome)))

    def test_go_to_root_must_open_the_root_directory(self):
        """Test that opens the Device bookmark from the places section."""
        self.app.main_view.go_to_place('placeDevice', None)

        folder_list_page = self.app.main_view.get_folder_list_page()
        self.assertThat(
            folder_list_page.get_current_path,
            Eventually(Equals('/')))
